package com.example.config;

import com.example.frame.utils.FileUtil;
import com.example.frame.utils.ZipUtil;
import com.example.module.admin.ent.Mapping;
import com.example.module.admin.ent.NginxCtrl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.system.ApplicationHome;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ClassPathResource;
import org.springframework.util.StringUtils;
import xyz.wbsite.wsqlite.ObjectClient;

import javax.annotation.PostConstruct;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.sql.SQLException;
import java.util.ArrayList;


@Configuration
public class NginxConfig {


    @Autowired
    private Environment environment;
    @Autowired
    private ObjectClient objectClient;

    @Bean
    public NginxCtrl initNginx() {
        String property = environment.getProperty("nginx-path");
        File nginxHome = null;
        if (StringUtils.isEmpty(property)) {
            ApplicationHome home = new ApplicationHome(getClass());
            // 当前运行jar文件
            File jarFile = home.getSource() != null ? home.getSource() : home.getDir();

            //jar同目录
            nginxHome = new File(jarFile.getParent(), "nginx");

            if (!nginxHome.exists()) {
                ClassPathResource classPathResource = new ClassPathResource("nginx-1.14.2.zip");
                try {
                    InputStream inputStream = classPathResource.getInputStream();
                    File nginxDir = new File(jarFile.getParent(), "nginx");
                    File nginxZip = new File(jarFile.getParent(), "nginx.zip");
                    FileUtil.inputStream2File(inputStream, nginxZip);
                    ZipUtil.unZip(nginxZip, nginxDir);
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        } else {
            nginxHome = new File(property);
        }

        if (!nginxHome.exists()) {
            throw new RuntimeException("nginx home not exists!");
        }

        File start = new File(nginxHome, "start.bat");
        File stop = new File(nginxHome, "stop.bat");
        File reload = new File(nginxHome, "reload.bat");

        NginxCtrl nginxCtrl = new NginxCtrl();
        nginxCtrl.setStartCmd(start.getAbsolutePath());
        nginxCtrl.setStopCmd(stop.getAbsolutePath());
        nginxCtrl.setReloadCmd(reload.getAbsolutePath());
        nginxCtrl.setVersionCmd(nginxHome.getAbsolutePath() + " -v ");

        return nginxCtrl;
    }
}
